@@ -19,6 +19,7 @@ from optax._src.alias import adabelief
 from optax._src.alias import adagrad
 from optax._src.alias import adam
 from optax._src.alias import adamw
+from optax._src.alias import dpsgd
 from optax._src.alias import fromage
 from optax._src.alias import lamb
 from optax._src.alias import noisy_sgd
@@ -30,10 +31,22 @@ from optax._src.combine import chain
 from optax._src.control_variates import control_delta_method
 from optax._src.control_variates import control_variates_jacobians
 from optax._src.control_variates import moving_avg_baseline
+from optax._src.linear_algebra import matrix_inverse_pth_root
+from optax._src.linear_algebra import power_iteration
+from optax._src.loss import cosine_distance
+from optax._src.loss import huber_loss
+from optax._src.loss import l2_loss
+from optax._src.loss import sigmoid_binary_cross_entropy
+from optax._src.loss import smooth_labels
+from optax._src.loss import softmax_cross_entropy
+from optax._src.privacy import differentially_private_aggregate
+from optax._src.privacy import DifferentiallyPrivateAggregateState
 from optax._src.schedule import constant_schedule
 from optax._src.schedule import cosine_decay_schedule
 from optax._src.schedule import cosine_onecycle_schedule
 from optax._src.schedule import exponential_decay
+from optax._src.schedule import inject_hyperparams
+from optax._src.schedule import InjectHyperparamsState
 from optax._src.schedule import linear_onecycle_schedule
 from optax._src.schedule import piecewise_constant_schedule
 from optax._src.schedule import piecewise_interpolate_schedule
@@ -108,7 +121,7 @@ from optax._src.wrappers import MaybeUpdateState
 from optax._src.wrappers import MultiSteps
 from optax._src.wrappers import MultiStepsState
 
-__version__ = "0.0.3"
+__version__ = "0.0.5"
 
 __all__ = (
     "adabelief",
@@ -137,7 +150,11 @@ __all__ = (
     "control_delta_method",
     "control_variates_jacobians",
     "cosine_decay_schedule",
+    "cosine_distance",
     "cosine_onecycle_schedule",
+    "dpsgd",
+    "differentially_private_aggregate",
+    "DifferentiallyPrivateAggregateState",
     "exponential_decay",
     "fisher_diag",
     "flatten",
@@ -145,14 +162,19 @@ __all__ = (
     "global_norm",
     "GradientTransformation",
     "hessian_diag",
+    "huber_loss",
     "hvp",
     "identity",
     "incremental_update",
+    "inject_hyperparams",
+    "InjectHyperparamsState",
     "lamb",
     "lookahead",
     "LookaheadParams",
     "LookaheadState",
+    "l2_loss",
     "linear_onecycle_schedule",
+    "matrix_inverse_pth_root",
     "masked",
     "MaskedState",
     "measure_valued_jacobians",
@@ -166,6 +188,7 @@ __all__ = (
     "piecewise_constant_schedule",
     "piecewise_interpolate_schedule",
     "polynomial_schedule",
+    "power_iteration",
     "radam",
     "rmsprop",
     "scale",
@@ -189,6 +212,9 @@ __all__ = (
     "Schedule",
     "score_function_jacobians",
     "sgd",
+    "sigmoid_binary_cross_entropy",
+    "smooth_labels",
+    "softmax_cross_entropy",
     "trace",
     "TraceState",
     "TransformInitFn",
