@@ -20,12 +20,17 @@ instance, they may be used to anneal the learning rate used to update an agent's
 parameters or the exploration factor used to select actions.
 """
 
-from typing import Callable, Dict, Union, Optional
+import functools
+import inspect
+from typing import Callable, Dict, Union, Optional, Iterable
 
 from absl import logging
 import chex
+import jax
 import jax.numpy as jnp
 
+from optax._src import transform
+from optax._src import utils
 
 Schedule = Callable[[chex.Numeric], chex.Numeric]
 
@@ -345,3 +350,120 @@ def cosine_onecycle_schedule(
       peak_value / div_factor,
       {int(pct_start * transition_steps): div_factor,
        int(transition_steps): 1. / (div_factor * final_div_factor)})
+
+
+def _convert_floats(x, dtype):
+  """Convert float-like inputs to dtype, rest pass through."""
+  if jax.dtypes.scalar_type_of(x) == float:
+    return jnp.asarray(x, dtype=dtype)
+  return x
+
+
+class InjectHyperparamsState(transform.OptState):
+  """Maintains inner transform state, hyperparameters, and step count."""
+  count: jnp.ndarray  # shape=(), dtype=jnp.int32
+  hyperparams: Dict[str, chex.Numeric]
+  inner_state: transform.OptState
+
+
+def inject_hyperparams(
+    inner_factory: Callable[..., transform.GradientTransformation],
+    static_args: Union[str, Iterable[str]] = ()
+) -> Callable[..., transform.GradientTransformation]:
+  """Wrapper that injects hyperparameters into the inner GradientTransformation.
+
+  This wrapper allows you to pass schedules (i.e. a function that returns a
+  numeric value given a step count) instead of constants for
+  hyperparameters. You may only schedule numeric hyperparameters (i.e. boolean
+  flags cannot be scheduled).
+
+  For example, to use `scale_by_adam` with a piecewise linear
+  schedule for beta_1 and constant for beta_2:
+
+  ```
+  scheduled_adam = optax.inject_hyperparams(optax.scale_by_adam)(
+      b1=optax.piecewise_linear_schedule(...),
+      b2=0.99)
+  ```
+
+  You may manually change numeric hyperparameters that were not scheduled
+  through the `hyperparams` dict in the InjectHyperparamState:
+
+  ```
+  state = scheduled_adam.init(params)
+  updates, state = scheduled_adam.update(grads, state)
+  state.hyperparams['b2'] = 0.95
+  updates, state = scheduled_adam.update(updates, state)  # uses b2 = 0.95
+  ```
+
+  Manually overriding scheduled hyperparameters will have no effect (e.g.
+  in the code sample above, you cannot manually adjust `b1`).
+
+  Args:
+    inner_factory: a function that returns the inner
+      `optax.GradientTransformation` given the hyperparameters.
+    static_args: a string or iterable of strings specifying which
+      callable parameters are not schedules. inject_hyperparams treats all
+      callables as schedules by default, so if a hyperparameter is a
+      non-schedule callable, you must specify that using this argument.
+
+  Returns:
+    A callable that returns a `optax.GradientTransformation`. This callable
+    accepts the same arguments as `inner_factor`, except you may provide
+    schedules in place of the constant arguments.
+  """
+  static_args = ({static_args} if isinstance(static_args, str) else
+                 set(static_args))
+  inner_signature = inspect.signature(inner_factory)
+
+  if not static_args.issubset(inner_signature.parameters):
+    raise ValueError(
+        '`static_args` must specify a subset of `inner_factory`\'s parameters. '
+        f'Given `static_args`: {static_args}. `inner_factory` parameters: '
+        f'{set(inner_signature.parameters.keys())}')
+
+  @functools.wraps(inner_factory)
+  def wrapped_transform(*args, **kwargs) -> transform.GradientTransformation:
+    bound_arguments = inner_signature.bind(*args, **kwargs)
+    bound_arguments.apply_defaults()
+
+    sched_hps, numeric_hps, other_hps = {}, {}, {}
+    for name, value in bound_arguments.arguments.items():
+      if name in static_args or isinstance(value, bool):
+        other_hps[name] = value
+      elif callable(value):
+        sched_hps[name] = value
+      elif isinstance(value, (int, float, chex.Array)):
+        numeric_hps[name] = value
+      else:
+        other_hps[name] = value
+
+    def schedule_fn(count, dtype):
+      return {k: _convert_floats(f(count), dtype) for k, f in sched_hps.items()}
+
+    def init_fn(params):
+      count = jnp.zeros([], jnp.int32)
+      dtype = getattr(next(iter(jax.tree_leaves(params)), None), 'dtype', None)
+      hparams = {
+          k: jnp.asarray(_convert_floats(v, dtype))
+          for k, v in numeric_hps.items()}
+      hparams.update(schedule_fn(count, dtype))
+      return InjectHyperparamsState(  # pylint:disable=too-many-function-args
+          count, hparams, inner_factory(**other_hps, **hparams).init(params))
+
+    def update_fn(updates, state, params=None):
+      count_inc = utils.safe_int32_increment(state.count)
+      dtype = getattr(next(iter(jax.tree_leaves(updates)), None), 'dtype', None)
+      hparams = {k: _convert_floats(v, dtype)
+                 for k, v in state.hyperparams.items()}
+      hparams.update(schedule_fn(count_inc, dtype))
+      updates, inner_state = inner_factory(**other_hps, **hparams).update(
+          updates, state.inner_state, params)
+
+      # pylint:disable=too-many-function-args
+      return updates, InjectHyperparamsState(count_inc, hparams, inner_state)
+      # pylint:enable=too-many-function-args
+
+    return transform.GradientTransformation(init_fn, update_fn)
+
+  return wrapped_transform
