@@ -16,10 +16,11 @@
 """Gradient transformations."""
 
 from typing import Any, Callable, NamedTuple, Optional, Sequence, Tuple, Union
+
 import chex
 import jax
 import jax.numpy as jnp
-from optax._src import schedule
+from optax._src import utils
 
 
 # pylint:disable=no-value-for-parameter
@@ -272,7 +273,8 @@ def _update_moment(updates, moments, decay, order):
       lambda g, t: (1 - decay) * (g ** order) + decay * t, updates, moments)
 
 
-def scale_by_rms(decay: float = 0.9, eps: float = 1e-8):
+def scale_by_rms(
+    decay: float = 0.9, eps: float = 1e-8, initial_scale: float = 0.):
   """Rescale updates by the root of the exp. moving avg of the square.
 
   References:
@@ -281,13 +283,15 @@ def scale_by_rms(decay: float = 0.9, eps: float = 1e-8):
   Args:
     decay: decay rate for the exponentially weighted average of squared grads.
     eps: term added to the denominator to improve numerical stability.
+    initial_scale: initial value for second moment
 
   Returns:
     An (init_fn, update_fn) tuple.
   """
 
   def init_fn(params):
-    nu = jax.tree_map(jnp.zeros_like, params)  # second moment
+    nu = jax.tree_map(
+        lambda n: jnp.full_like(n, initial_scale), params)  # second moment
     return ScaleByRmsState(nu=nu)
 
   def update_fn(updates, state, params=None):
@@ -307,7 +311,8 @@ class ScaleByRStdDevState(OptState):
 
 
 def scale_by_stddev(
-    decay: float = 0.9, eps: float = 1e-8) -> GradientTransformation:
+    decay: float = 0.9, eps: float = 1e-8,
+    initial_scale: float = 0.) -> GradientTransformation:
   """Rescale updates by the root of the centered exp. moving average of squares.
 
   References:
@@ -316,6 +321,7 @@ def scale_by_stddev(
   Args:
     decay: decay rate for the exponentially weighted average of squared grads.
     eps: term added to the denominator to improve numerical stability.
+    initial_scale: initial value for second moment
 
   Returns:
     An (init_fn, update_fn) tuple.
@@ -323,7 +329,8 @@ def scale_by_stddev(
 
   def init_fn(params):
     mu = jax.tree_map(jnp.zeros_like, params)  # First moment
-    nu = jax.tree_map(jnp.zeros_like, params)  # Second moment
+    nu = jax.tree_map(
+        lambda n: jnp.full_like(n, initial_scale), params)  # second moment
     return ScaleByRStdDevState(mu=mu, nu=nu)
 
   def update_fn(updates, state, params=None):
@@ -345,22 +352,8 @@ class ScaleByAdamState(OptState):
   nu: Updates
 
 
-def _safe_int32_increment(count):
-  """Increments int32 counter by one.
-
-  Normally `max_int + 1` would overflow to `min_int`. This functions ensures
-  that when `max_int` is reached the counter stays at `max_int`.
-
-  Args:
-    count: a counter to be incremented.
-
-  Returns:
-    a counter incremented by 1, or max_int if the maximum precision is reached.
-  """
-  chex.assert_type(count, jnp.int32)
-  max_int32_value = jnp.iinfo(jnp.int32).max
-  one = jnp.array(1, dtype=jnp.int32)
-  return jnp.where(count < max_int32_value, count + one, max_int32_value)
+# TODO(b/183478923): remove legacy reference.
+_safe_int32_increment = utils.safe_int32_increment
 
 
 def _bias_correction(moment, decay, count):
@@ -398,7 +391,7 @@ def scale_by_adam(b1: float = 0.9,
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
     nu = _update_moment(updates, state.nu, b2, 2)
-    count_inc = _safe_int32_increment(state.count)
+    count_inc = utils.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
     nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
@@ -469,7 +462,7 @@ def scale_by_belief(
     mu = _update_moment(updates, state.mu, b1, 1)
     prediction_error = jax.tree_multimap(lambda g, m: g-m, updates, state.mu)
     nu = _update_moment(prediction_error, state.nu, b2, 2)
-    count_inc = _safe_int32_increment(state.count)
+    count_inc = utils.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
     nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
@@ -513,7 +506,7 @@ def scale_by_yogi(
     signed_sq = jax.tree_multimap(
         lambda g, v: jnp.sign(v - g**2)*g**2, updates, state.nu)
     nu = _update_moment(signed_sq, state.nu, b2, 2)
-    count_inc = _safe_int32_increment(state.count)
+    count_inc = utils.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
     nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
@@ -564,7 +557,7 @@ def scale_by_radam(b1: float = 0.9,
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
     nu = _update_moment(updates, state.nu, b2, 2)
-    count_inc = _safe_int32_increment(state.count)
+    count_inc = utils.safe_int32_increment(state.count)
     b2t = b2**count_inc
     ro = ro_inf - 2 * count_inc * b2t / (1 - b2t)
     mu_hat = _bias_correction(mu, b1, count_inc)
@@ -614,7 +607,7 @@ class ScaleByScheduleState(OptState):
   count: jnp.ndarray  # shape=(), dtype=jnp.int32
 
 
-def scale_by_schedule(step_size_fn: schedule.Schedule):
+def scale_by_schedule(step_size_fn: Callable[[chex.Numeric], chex.Numeric]):
   """Scale updates using a custom schedule for the `step_size`.
 
   Args:
@@ -634,7 +627,7 @@ def scale_by_schedule(step_size_fn: schedule.Schedule):
     updates = jax.tree_map(
         lambda g: jnp.array(step_size, dtype=g.dtype) * g, updates)
     return updates, ScaleByScheduleState(
-        count=_safe_int32_increment(state.count))
+        count=utils.safe_int32_increment(state.count))
 
   return GradientTransformation(init_fn, update_fn)
 
@@ -644,24 +637,6 @@ class ScaleByFromageState(OptState):
   count: jnp.ndarray  # shape=(), dtype=jnp.int32
 
 
-def _safe_norm(x, min_norm):
-  """Returns jnp.maximum(jnp.linalg.norm(x), min_norm) with correct gradients.
-
-  The gradients of jnp.maximum(jnp.linalg.norm(x), min_norm) at 0.0 is NaN,
-  because jax will evaluate both branches of the jnp.maximum.
-
-  The version in this function will return the correct gradient of 0.0 in this
-  situation.
-
-  Args:
-    x: jax array.
-    min_norm: lower bound for the returned norm.
-  """
-  norm = jnp.linalg.norm(x)
-  x = jnp.where(norm < min_norm, jnp.ones_like(x), x)
-  return jnp.where(norm < min_norm, min_norm, jnp.linalg.norm(x))
-
-
 class ScaleByTrustRatioState(NamedTuple):
   """The scale and decay trust ratio transformation is stateless."""
 
@@ -689,8 +664,8 @@ def scale_by_trust_ratio(min_norm: float = 0.0) -> GradientTransformation:
     def _scale_update(update, param):
 
       # Clip norms to minimum value, by default no clipping.
-      param_norm = _safe_norm(param, min_norm)
-      update_norm = _safe_norm(update, min_norm)
+      param_norm = utils.safe_norm(param, min_norm)
+      update_norm = utils.safe_norm(update, min_norm)
       trust_ratio = param_norm / update_norm
 
       # If no minimum norm clipping is used
@@ -736,7 +711,7 @@ def add_noise(eta: float, gamma: float, seed: int) -> GradientTransformation:
     del params
     num_vars = len(jax.tree_leaves(updates))
     treedef = jax.tree_structure(updates)
-    count_inc = _safe_int32_increment(state.count)
+    count_inc = utils.safe_int32_increment(state.count)
     variance = eta / count_inc**gamma
     all_keys = jax.random.split(state.rng_key, num=num_vars + 1)
     noise = jax.tree_multimap(
@@ -784,7 +759,7 @@ def apply_every(k: int = 1) -> GradientTransformation:
         lambda g, ga: acc * ga + g, updates, state.grad_acc)
     emit = c == (k - 1)
     updates = jax.tree_map(lambda ga: emit * ga, grad_acc)
-    count_inc = _safe_int32_increment(state.count)
+    count_inc = utils.safe_int32_increment(state.count)
     return updates, ApplyEvery(count=count_inc % k, grad_acc=grad_acc)
 
   return GradientTransformation(init_fn, update_fn)
